function varargout = Gammachirp_Basilar_membrane(varargin)

%%%%%%Dr. Vicente Sierra-Vázquez & Dr. Ignacio Serrano-Pedraza 2022
%%%iserrano@ucm.es

%MARZO 2022
%%Complex Gammachirp-based (auditory) TFR

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Analyzing function: Complex gammachirp filter maxMODFT=1, i.e. Area=1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%I-INTRODUCTION :
%From Irino & Patterson [1,2]: The gammatone auditory filter fits well the empirical properties of
%the cochlear impulse responses and its MTF was in good agreement with the magnitude
%response of auditory filter models used to fit a wide range of human
%masking data at moderate sound levels. However, the gammatone filter, with a symmetrical MTF,
%does not capture some kown empirical facts: the asymmmetric envelope
%of the Basilar Membrane Motion when you hear a tone, the increasing asymmetry of the response magnitude
%of auditory filters with the sound level, and the data of asymmetries of tone masking.
%To overcome partially those
%drawbacks it is derived [1] an AM-FM analytic function
%composed by the AM envelope from the gammatone filters and a FM carrier (a
%chirp), and hence the gammachirp name. (Note in the corresponding Demos the
%asymmetric MTFs in the gammachirp filterbanks).


%IV-FUNDAMENTALS on the complex Gammachirp filter

%A-Definition
%(i): Impulse response function
%The impulse response function of the complex gammachirp (eq (1) of [2]) is
%an AM-FM function defined as

% g(t,n)=[A*t^(a-1)*exp(-2pi*b*t)]*exp(j2pi*f*t+phi)*exp(j*c*ln(t))*U(t)

%g(t,n)=[A*t^(a-1)*exp(-2pi*b*t)]*exp(j2pi*f*t)*(t^(jc))*U(t)

% where A, amplitude, a normalization factor,
% if a in N, a=n, n is order of filter, (for auditory filters n=4)
%b (envelope spread),
%f, Hz, asymtotic temporal frequency, and
%b=1.019*ERB(f) (for n=4) (see B- )
%c=-2 (experimental datum)
%U the Heaviside unit step function
%and ERB is the equivalent rectangular bandwidth of the filter at moderate levels.
%ERB(f)= 24.7*(0.00437*f+1) in Hz (Glasberg and Moore, 1990)[6] .
%Warning: our b=b1*ERB(f) of eq. 1 in [2]).

%The filter gets its name from the fact that the AM envelope formed
%by the power function and the exponential (in brackets in the formula)
%is a gamma distribution function, and the complex exponential carrier is a FM chirp whose
%real part (or imaginary part) is in the auditory range.


%(i)FT of the impulse response
%The MTF is

%MOD(g^(xi))=A*(abs(gammac(a+jc))/(2*pi*sqrt(b^2+(xi-f)^2))^a)*exp(c*arctan((xi-f)/b))

%The PTF, phi,is
% phiGT=-a*atan((xi-f)/b)
% phiGC1=-c*((1/2)*log(4*pi^2*(b^2+(xi-f)^2)))
% phuGC2=arctan(Im(gammc(a+jc))/Re(gammac(a+jc)))
% phi(xi)=phiGT+phiGC1+phiGC2

%the peak frequency of MTF as a function of the asymtotic temporal
%frequency f, c, b and a is

%fP=f+cb/a

%The normalization factor A for maxMODFT=1 is

%A=((2*pi*sqrt^(b^2*(1+(c/a)^2)))^a)/(abs(gammac(a+jc))*exp(-c*arctan(c/a)))

%(ii)Complex Gammachirp (auditory) transform (CGT)

%Because g is the impulse response, the CGT of a signal x(t), Gx, is

% Gx(tau,f)=x*g
%  =INT(-inf)(inf)x(t)[A*t^(n-1)*exp(-2pi*b*(tau-t))]*exp(j2pi*f*(tau-t)t+phi)*exp(j*c*ln(tau-t))

%Gx(tau,f) is a complex-valued function so
%Gx(tau,f)=Re(Gx(tau,f))+iIm(Gx(tau,f)),
%and, alternatively,
%Gx(tau,f)=mod(Gx(tau,f))*exp(i*phi(tau,f))
%where mod(Gx(tau,f))>=0 is the CGT magnitude and phi(tau,f), -pi<phi<=pi,
%is the GGT phase

%The magnitude or squared magnitude of TF Gammatone Representation of a sound is called
%"Gammachirp spectrogram" of the sound.

%Note that the Basilar Membrane undulation (BMM) at tau moment when you hear the sound x(t)
%is represented by the Real part of Gx as a function of f at the tau instant
%i.e.,

%BMMx {tau}(f)=Re(Gx(tau;f))

%The Basilar Membrane Motion (compare the Demo with Fig 1(a) in [7]) is the of BMN(f) at each instant tau and the
%temporal evolution of the evelope is

%Envx {tau}(f)=mod(Gx(tau,f))

%Note finally that a the gammachirp filterbank generates an asymmerical envelope for BMMs for a tone,
%contrary to the symmetrical one with the gammatone filter.

%References:
%[1]T. Irino and R. D. Paterson,"A time-domain, level-dependent auditory filter: The gammachirp,"
%J. Acoust. Soc. Am. vol. 101, no.1, pp.412-419 Jan 1997.
%[2]T. Irino and R. D. Paterson, "A compressive gammachirp auditory filter for both physiological and
%psychophysical data," J. Acoust. Soc. Am. vol. 109 no.5, pp.2008-2022, May
%2001.


%%%For the cochlear frequency-position we have used the following equation from Greenwood (1990):
%     x=linspace(0,25,NF);%%%25 mm corresponds to 5084.85Hz
%     A=165.4;
%     a=0.06;%%x is expressed in millimeters
%     %a=2.1; %if x is expressed as a proportion of the basilar membrane
%     k=0.88;%%Assuming that 35 mm corresponds to 20.000 Hz and 0 mm to 20Hz
%     u11=A*(10.^(a*x)-k);%%Greenwood 1990

%%%Greenwood, D.D. (1990). "A cochlear frequency-position function for several species – 29 years later". Journal of the Acoustical Society of America. 87 (6): 2592–2605.




% GAMMACHIRP_BASILAR_MEMBRANE MATLAB code for Gammachirp_Basilar_membrane.fig
%      GAMMACHIRP_BASILAR_MEMBRANE, by itself, creates a new GAMMACHIRP_BASILAR_MEMBRANE or raises the existing
%      singleton*.
%
%      H = GAMMACHIRP_BASILAR_MEMBRANE returns the handle to a new GAMMACHIRP_BASILAR_MEMBRANE or the handle to
%      the existing singleton*.
%
%      GAMMACHIRP_BASILAR_MEMBRANE('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in GAMMACHIRP_BASILAR_MEMBRANE.M with the given input arguments.
%
%      GAMMACHIRP_BASILAR_MEMBRANE('Property','Value',...) creates a new GAMMACHIRP_BASILAR_MEMBRANE or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before Gammachirp_Basilar_membrane_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to Gammachirp_Basilar_membrane_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help Gammachirp_Basilar_membrane

% Last Modified by GUIDE v2.5 09-Mar-2022 17:44:04

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @Gammachirp_Basilar_membrane_OpeningFcn, ...
    'gui_OutputFcn',  @Gammachirp_Basilar_membrane_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% --- Executes just before Gammachirp_Basilar_membrane is made visible.
function Gammachirp_Basilar_membrane_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to Gammachirp_Basilar_membrane (see VARARGIN)

% Choose default command line output for Gammachirp_Basilar_membrane
handles.output = hObject;
% % Play_sound handles structure
guidata(hObject, handles);
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)


function Plot_figure(hObject, eventdata, handles,playsound)
cla
frecuenciamuestreo=4096;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NH1=3441;
fileID=fopen('DATA/H1data_GWobserved_sc.txt','r');
C_H1 = textscan(fileID,'%f %f',NH1);
fclose(fileID);
time_H1=C_H1{1}';
senalGW_H1=C_H1{2}';
senalH1=[time_H1;senalGW_H1];
%duracion real de la GW
xinicio=min(time_H1);
xend1=0.4599609375;
tfinal=max(time_H1);
xdur=tfinal-xinicio;
xinct=1./16384;
NH1=3441;
%control
xdur=xinct.*(NH1-1);
%para evitar los problemas de la convolucion circular
NC=4096;
xtamt=xdur.*(NC-1)./(NH1-1);
%control
xtamt=xinct.*(NC-1);
tmin1=xinicio-328.*xinct;
tmax1=tmin1+xtamt;
t=linspace(tmin1,tmax1,NC);
duracion=tmax1-tmin1;



loadsignals_index = get(handles.loadsignals, 'Value');

switch loadsignals_index
    
    case 1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       Gravitational wave H1
fileID=fopen('DATA/H1data_GWobserved_sc.txt','r');
C_H1 = textscan(fileID,'%f %f',NH1);
fclose(fileID);
time_H1=C_H1{1}';
senalGW_H1=C_H1{2}';
senalH1=[time_H1;senalGW_H1];
%duracion real de la GW
xinicio=min(time_H1);
xend1=0.4599609375;
tfinal=max(time_H1);
xdur=tfinal-xinicio;
xinct=1./16384;
NH1=3441;
%control
xdur=xinct.*(NH1-1);
%para evitar los problemas de la convolucion circular
NC=4096;
xtamt=xdur.*(NC-1)./(NH1-1);
%control
xtamt=xinct.*(NC-1);
tmin1=xinicio-328.*xinct;
tmax1=tmin1+xtamt;
t=linspace(tmin1,tmax1,NC);



%Extended to NC
H1GW=zeros(1,NC);
 
for k=1:NC
    if(k<=328)
        H1GW(k)=0;
    end
    if(k>=329&&k<=329+NH1-1)
        j=k-328;
        H1GW(k)=senalGW_H1(j);
    end
    if(k>329+NH1-1)
        H1GW(k)=0;
    end
end

%Extended gravitational wave H1 
senalGW=H1GW;
size(senalGW);

senalH1_ext=[t;senalGW];
FS=4096;
YY=1*senalGW';

        
   color=[1 0 0];        
        
    case 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  H1 relativity template

fileID=fopen('DATA/H1data_GWtemplate_sc.txt','r');
C = textscan(fileID,'%f %f',NH1);
fclose(fileID);

time_H1NR=C{1}';
templateGW_H1NR=C{2}';
templateH1=[time_H1NR;templateGW_H1NR];

xinicio1=min(time_H1NR);
xend1=max(time_H1NR);
tfinal1=max(time_H1NR);
xdur1=tfinal1-xinicio1;
xinct=1./16384;

NH1NR=3440;

%Extended to NC
H1NR=zeros(1,NC);
t_H1NR=zeros(1,NC);
for k=1:NC
    if(k<=328)
        H1NR(k)=0;
        t_H1NR(k)=t(k);
    end
    if(k>=329&k<=329+NH1NR-1)
        j=k-328;
        H1NR(k)=templateGW_H1NR(j);
        t_H1NR(k)=time_H1NR(j);
    end
    if(k>329+NH1NR-1)
        H1NR(k)=0;
        t_H1NR(k)=t(k);
    end
end

templateGWR=H1NR;

templateH1_ext=[t_H1NR;templateGWR];

FS=4096;
YY=1*templateGWR';    
color=[1 0 0];      
    
    case 3
%    My reconstructed H1

 [time H1GW_VSV]...
     =textread('DATA/H1rec_main_VSV.txt','%n%n');

GWH1_rec=H1GW_VSV;
FS=4096;
YY=1*GWH1_rec; 
color=[1 0 0];      

    case 4
        
%Gravitational wave L1
NL1=3441;
fileID=fopen('DATA/L1data_GWobserved_sc.txt','r');
C = textscan(fileID,'%f %f',NL1);
fclose(fileID);

time_L1=C{1}';
senalGW_L1=C{2}';
senalL1=[time_L1;senalGW_L1];
%Extended to NC
L1GW=zeros(1,NC);
 
for k=1:NC
    if(k<=328)
        L1GW(k)=0;
    end
    if(k>=329&k<=329+NL1-1)
        j=k-328;
        L1GW(k)=senalGW_L1(j);
    end
    if(k>329+NL1-1)
        L1GW(k)=0;
    end
end

senalGW=L1GW;

FS=4096;
YY=1*senalGW';
       
   color=[0 0 1];  

    case 5
%%%  relativity template L1
NL1=3441;
fileID=fopen('DATA/L1data_GWtemplate_sc.txt','r');
C = textscan(fileID,'%f %f',NL1);
fclose(fileID);

time_L1NR=C{1}';
templateGW_L1NR=C{2}';
templateL1=[time_L1NR;templateGW_L1NR];

xinicio2=min(time_L1NR);
xend2=max(time_L1NR);
tfinal2=max(time_L1NR);
xdur2=tfinal2-xinicio2;
xinct=1./16384;

NL1NR=3440;

%Extended to NC
L1NR=zeros(1,NC);
t_L1NR=zeros(1,NC);

for k=1:NC
    if(k<=328)
        L1NR(k)=0;
        t_L1NR(k)=t(k);
    end
    if(k>=329&&k<=329+NL1NR-1)
        j=k-328;
        L1NR(k)=templateGW_L1NR(j);
        t_L1NR(k)=time_L1NR(j);
    end
    if(k>329+NL1NR-1)
        L1NR(k)=0;
        t_L1NR(k)=t(k);
    end
end

templateGWR=L1NR;
size(templateGWR);

templateL1_ext=[t_L1NR;templateGWR];
FS=4096;
YY=1*templateGWR';    
color=[0 0 1];    

    case 6
%         My reconstructed L1

[time L1GW_VSV]...
     =textread('DATA/L1rec_main_VSV.txt','%n%n');

GWL1_rec=L1GW_VSV;

FS=4096;
YY=1*GWL1_rec; 
color=[0 0 1];   
 
    case 7 %% H1NR synthetic ring-down
        
[t Ig_7QNM_V2]...
     =textread('DATA/Ig_7QNM_V2.txt','%n%n');
        

YY=Ig_7QNM_V2; 
color=[0 0 1];   
tmax1=max(t);
tmin1=min(t);
duracion=tmax1-tmin1;
FS=length(t)/duracion;
color=[1 0 0]; 

end


if get(handles.peakfreqbutton,'Value')==0 && get(handles.asymfreq,'Value')==0  && get(handles.stapes,'Value')==0
    set(handles.asymfreq,'Value',1)
end


%%Color maps

load('modulo_Ignacio_25');%%Modulo (I. Serrano-Pedraza, based on CIE1931 and converting XYZ values to sRGB)


%fase (colour circle) (Vicente) -pi (blue) -pi/2 (green) 0 (yellow) pi/2 (red) + pi (blue)
r([1:64])=0;r([65:128])=[0:1/63:1];r([129:192])=1;r([193:255])=[1-1/63:-1/63:0];
g([1:64])=[0:1/63:1];g([65:128])=1;g([129:192])=[1:-1/63:0];g([193:255])=0;
b([1:64])=[1:-1/63:0];b([65:128])=0;b([129:192])=0;b([193:255])=[0:1/63:1-1/63];
colourPh=[r',g',b'];

%%%%%%%Grayscale
r=linspace(0,1,255);
g=linspace(0,1,255);
b=linspace(0,1,255);
gray2=[r',g',b'];


umin=0;
umax=500;
tmax=tmax1;%duracion;
tmin=tmin1;
xTAMt=duracion;
NF=128;%%Vertical Resolution of the Gammatonegram
NC=length(YY);
t=linspace(tmin,tmax,NC);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


axes(handles.Plot1);
cla;
plot(t,YY,'Color',color,'LineWidth',1.25);
axis([tmin tmax -1.2 1.2])
h=gca;
h.YAxis.MinorTick = 'on';
 labels=linspace(tmin,tmax,5);
 T = linspace(tmin,tmax,5);
 str=num2str(T','%10.3f');
 set(gca,'xTick',labels,'xTickLabel',{str})
 title('Signal')
 xlabel('Time (sec)')
 ylabel('Strain (10^{-21})')
set(gca,'TickDir','out');
a = gca;
set(a,'box','off','color','none');
b = axes('Position',get(a,'Position'),'box','on','xtick',[],'ytick',[]);
axes(a)




if playsound==1
    sound(0.8*YY,frecuenciamuestreo)
end



axes(handles.Plot2);
cla;

Y=fftshift(fft(YY'));%%FFT of the signal
xTAMt=duracion;
n=length(YY);
tt=t;
tt=[(-NC/2):1:NC/2-1]*(xTAMt/(NC-1));

if get(handles.stapes,'Value')==1
    x=linspace(0,25,NF);%%%25 mm corresponds to 5084.85Hz
    A=165.4;
    a=0.06;%%x is expressed in millimeters
    %a=2.1; %if x is expressed as a proportion of the basilar membrane
    k=0.88;%%Assuming that 35 mm corresponds to 20.000 Hz and 0 mm to 20Hz
    u11=A*(10.^(a*x)-k);%%Greenwood 1990
else
    u11=linspace(0,umax,NF);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Parameters for the filterbank
nn=4;
cc=str2double(get(handles.cc,'String'));
t00=0;

for i=1:NF
    
    
    
    %%%%%%%%COMPlEX GAMMACHIRP
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Computation of bb and the normalization factor (maxMOD=1)
    
    if get(handles.ERB_Munich,'value')==1
        ERB_Mun=25+75*(1+1.4*(u11(i)/1000).^2).^0.69;
        bb=1.019*ERB_Mun;
    end
    if get(handles.ERB_Cambridge,'value')==1
        ERB_Cam=24.7*(4.37*(u11(i)/1000)+1);
        bb=1.019*ERB_Cam;
    end
    if (get(handles.ERB_Cambridge,'value')==0 && get(handles.ERB_Munich,'value')==0)
        ERB_Cam=24.7*(4.37*(u11(i)/1000)+1);
        bb=1.019*ERB_Cam;
    end
    
    
    %Gammachirp
    z=nn+sqrt(-1)*cc;
    coefnorm=(2*pi*sqrt(bb^2*(1+(cc/nn)^2)))^nn*(abs(gammac(z)))^(-1)*(exp(cc*atan(cc/nn)))^(-1);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    filtro_R=zeros(1,NC);
    filtro_I=zeros(1,NC);
    for j=1:NC
        if(j>=NC/2+2)
            filtro_R(j)=coefnorm.*((tt(j)-t00).^(nn-1)).*exp(-(tt(j)-t00).*2*pi*bb).*cos(2.*pi*u11(i)*(tt(j)-t00)+cc*log(tt(j)));%%Gammachirp I & P (2001)
            filtro_I(j)=coefnorm.*((tt(j)-t00).^(nn-1)).*exp(-(tt(j)-t00).*2*pi*bb).*sin(2.*pi*u11(i)*(tt(j)-t00)+cc*log(tt(j)));
        end
    end
    
    
    
    filtro=(filtro_R+filtro_I*sqrt(-1));
    
    FACT=xTAMt/(NC);
    FFT_filtro=fftshift(fft(filtro)).*FACT;
    modulo_filtro=abs(FFT_filtro)';
    fase_filtro=angle(FFT_filtro)';
    
    
    modulo=abs(Y)';
    fase=angle(Y)';
    REALPROD=fftshift(modulo.*modulo_filtro.*cos(fase+fase_filtro));
    IMAGPROD=fftshift(modulo.*modulo_filtro.*sin(fase+fase_filtro));
    RESULTADO=(REALPROD+IMAGPROD*sqrt(-1));
    FINAL=fftshift(ifft(RESULTADO));
    yR=real(FINAL);
    yI=imag(FINAL);
    modulo_final(:,i)=sqrt(yR.^2+yI.^2);
    phase_final(:,i)=atan2(yI,yR);
    yR_final(:,i)=yR;
    
end

modulo_final_norm= 255*flipud(modulo_final'./max(max(modulo_final)));
phase_final_norm= 255*(1+flipud(phase_final'./max(max(abs(abs(phase_final))))))./2;
yR_final_norm= 127*(1+flipud(yR_final'./max(max(yR_final))));



%%%%
if (get(handles.realpart,'Value')==0 && get(handles.magnitude,'Value')==0)
    set(handles.magnitude,'Value',1);
end
if get(handles.realpart,'Value')==1
    image(yR_final_norm)
    colormap(handles.Plot2,gray2);
end
if get(handles.magnitude,'Value')==1
    image(modulo_final_norm)
    colormap(handles.Plot2,colour3);
end
hcb = colorbar('Location','EastOutside','XTickMode','manual');
hcb.Label.String='Amplitude';
labels=linspace(1,255,5);
AA = [0 0.25 0.5 0.75 1]';
str=num2str(AA,'%10.2f');
set(hcb,'YTick',labels,'YTickLabel',{str})

labels=linspace(1,NC,5);
T = linspace(tmin,tmax,5)';
str=num2str(T,'%10.2f');
set(gca,'xTick',labels,'xTickLabel',{str})



FF1= [50:50:500];
f00=FF1;
%Computation and b and the normalization factor (maxMOD=1)
if get(handles.ERB_Munich,'value')==1
    ERB_Mun=25+75*(1+1.4*(f00/1000).^2).^0.69;
    bb=1.019*ERB_Mun;
end
if get(handles.ERB_Cambridge,'value')==1
    ERB_Cam=24.7*(4.37*(f00/1000)+1);
    bb=1.019*ERB_Cam;
end
if (get(handles.ERB_Cambridge,'value')==0 && get(handles.ERB_Munich,'value')==0)
    ERB_Cam=24.7*(4.37*(f00/1000)+1);
    bb=1.019*ERB_Cam;
end


f00=FF1-cc*bb/nn;%%%Irino & Patterson (2001) equation 5.
FP1=f00;

if get(handles.asymfreq,'Value')==1
    labels1=fliplr(NF-[0:50:umax]*NF./umax);
    labels1(1)=1;
    F1 = [umax:-50:0]';
end
if get(handles.peakfreqbutton,'Value')==1
    labels1=fliplr(NF-FP1*NF./umax);

        F1=[umax:-50:50]';

end
if get(handles.stapes,'Value')==1
    labels1=linspace(1,NF,6);
    F1=linspace(25,0,6)';
end


str=num2str(F1,'%10.0f');
set(gca,'yTick',labels1,'yTickLabel',{str})
grid

xlabel('Time (sec)')
if get(handles.asymfreq,'Value')==1
    ylabel('Asymptotic Temporal Frequency (Hz)')
end
if get(handles.peakfreqbutton,'Value')==1
    ylabel('Peak Temporal Frequency (Hz)')
end
if get(handles.stapes,'Value')==1
    ylabel('Distance from the Apex (mm)')
end
if get(handles.realpart,'Value')==1
    title('Time-frequency real part')
else
    title('Time-frequency magnitude')
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if get(handles.Run,'Value')==1
    axes(handles.Plot5);
    %cla;
    set(handles.Apex,'Visible','on')
    set(handles.Base,'Visible','on')
    set(handles.text64,'Visible','on')

    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    FF1= [50:50:500];
    f00=FF1;
    %Computation and b and the normalization factor (maxMOD=1)
    if get(handles.ERB_Munich,'value')==1
        ERB_Mun=25+75*(1+1.4*(f00/1000).^2).^0.69;
        bb=1.019*ERB_Mun;
    end
    if get(handles.ERB_Cambridge,'value')==1
        ERB_Cam=24.7*(4.37*(f00/1000)+1);
        bb=1.019*ERB_Cam;
    end
    if (get(handles.ERB_Cambridge,'value')==0 && get(handles.ERB_Munich,'value')==0)
        ERB_Cam=24.7*(4.37*(f00/1000)+1);
        bb=1.019*ERB_Cam;
    end
    
 %   FP=f00+cc*bb/nn; %%%Irino & Patterson (2001) equation 5.
f00=FF1-cc*bb/nn;
FP1=f00;
    if get(handles.asymfreq,'Value')==1
        labels1=[0:50:500];
        F1 = [0:50:500]';
    end
    
    if get(handles.peakfreqbutton,'Value')==1
        labels1=FP1;
        if cc>=1
            F1=[50:50:umax]';
        else
            F1=[50:50:umax]';
        end
    end
    if get(handles.stapes,'Value')==1
        labels1=linspace(0,25,6);
        F1=linspace(0,25,6)';
        u11=linspace(0,25,length(u11));
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    
    if get(handles.peakfreqbutton,'Value')==1
    f00=u11;
    %Computation and b and the normalization factor (maxMOD=1)
    if get(handles.ERB_Munich,'value')==1
        ERB_Mun=25+75*(1+1.4*(f00/1000).^2).^0.69;
        bb=1.019*ERB_Mun;
    end
    if get(handles.ERB_Cambridge,'value')==1
        ERB_Cam=24.7*(4.37*(f00/1000)+1);
        bb=1.019*ERB_Cam;
    end
    if (get(handles.ERB_Cambridge,'value')==0 && get(handles.ERB_Munich,'value')==0)
        ERB_Cam=24.7*(4.37*(f00/1000)+1);
        bb=1.019*ERB_Cam;
    end
    
    u12=f00+cc*bb/nn; %%%Irino & Patterson (2001) equation 5.
    end
    
    t=linspace(tmin,tmax,NC);
    H=find(t>=tmin);
    J=find(t<=tmax);
    dif=J(end)-H(1);
    MAXENV=max(max(modulo_final));
    
    for i=800:25:dif-100
        yR_final1=yR_final';
        BM_motion_1=yR_final1(:,H(1)+i);
        modulo_final1=modulo_final';
        Envelope=modulo_final1(:,H(1)+i);
        
        
        if get(handles.temporalpos,'Value')==1
            axes(handles.Plot5);
        end
        plot(u11,BM_motion_1,'b-',u11,Envelope,'r-',u11,0*Envelope,'k-.')%,u11(h)*ones(1,length(y)),y,'g-.')
        hh=find(Envelope==max(Envelope));
        if get(handles.temporalpos,'Value')==1
        x=u11(hh);
        if get(handles.peakfreqbutton,'Value')==1
        x=u12(hh);
        end
        y=max(Envelope)+0.05;
        text(x,y,['(',num2str(x),')'])
        end
        
        str=num2str(F1,'%10.0f');
        set(gca,'xTick',labels1,'xTickLabel',{str})
        grid on
        if get(handles.asymfreq,'Value')==1
        xlabel('Asymptotic Temporal Frequency (Hz)')
        end
        if get(handles.peakfreqbutton,'Value')==1
        xlabel('Peak Temporal Frequency (Hz)')
        end
        if get(handles.stapes,'Value')==1
            xlabel('Distance from the Apex (mm)')
            umax=25;
        end
        ylabel('Amplitude')
       
            axis([0 umax -(MAXENV+0.3*MAXENV) (MAXENV+0.3*MAXENV)])
        
        
        if get(handles.temporalpos,'Value')==1
            axes(handles.Plot1);
            cla;
            if color(1)==1
            plot(t,YY,'r-',t(i),YY(i),'go',t(i)*ones(1,10),linspace(-1.2,1.2,10),'g-','LineWidth',1.25);
            end
            if color(3)==1
            plot(t,YY,'b-',t(i),YY(i),'go',t(i)*ones(1,10),linspace(-1.2,1.2,10),'g-','LineWidth',1.25);
            end
            axis([tmin tmax -1.2 1.2])
            h=gca;
            h.YAxis.MinorTick = 'on';
            set(gca,'TickDir','out');
            set(gca,'box','off','color','none');
            title('Signal')
            xlabel('Time (sec)')
            ylabel('Strain (10^{-21})')

        end

        pause(0.001)
    end
else
    axes(handles.Plot5);
    cla;
    set(handles.Plot5,'Visible','off')
    set(handles.Apex,'Visible','off')
    set(handles.Base,'Visible','off')
    set(handles.text64,'Visible','off')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




% --- Outputs from this function are returned to the command line.
function varargout = Gammachirp_Basilar_membrane_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;

% --- Executes on button press in Play_sound.
function Play_sound_Callback(hObject, eventdata, handles)
% hObject    handle to Play_sound (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
playsound=1;
Plot_figure(hObject, eventdata, handles,playsound)



% --------------------------------------------------------------------
function OpenMenuItem_Callback(hObject, eventdata, handles)
% hObject    handle to OpenMenuItem (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
file = uigetfile('*.fig');
if ~isequal(file, 0)
    open(file);
end



function expt = getexpt(handles)
% Reads off key data from GUI

% --- Executes on button press in Plot_figure.
function Plot_figure_Callback(hObject, eventdata, handles)
% hObject    handle to Plot_figure (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)


function CloseMenuItem_Callback(hObject, eventdata, handles)
% hObject    handle to CloseMenuItem (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
selection = questdlg(['Close ' get(handles.axixX,'Name') '?'],...
    ['Close ' get(handles.axixX,'Name') '...'],...
    'Yes','No','Yes');
if strcmp(selection,'No')
    return;
end

delete(handles.axixX)
% --------------------------------------------------------------------
function PrintMenuItem_Callback(hObject, eventdata, handles)
% hObject    handle to PrintMenuItem (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
printdlg(handles.axixX)

% --------------------------------------------------------------------
function FileMenu_Callback(hObject, eventdata, handles)
% hObject    handle to FileMenu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in Reset.
function Reset_Callback(hObject, eventdata, handles)
% hObject    handle to Reset (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)




set(handles.ERB_Cambridge,'Value',1);
set(handles.ERB_Munich,'Value',0);
set(handles.cc,'String', -2);
set(handles.temporalpos,'Value',0);
set(handles.asymfreq,'Value',1);
set(handles.peakfreqbutton,'Value',0);
set(handles.stapes,'Value',0);
set(handles.realpart,'Value',0)



playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)





% --- Executes on button press in ERB_Cambridge.
function ERB_Cambridge_Callback(hObject, eventdata, handles)
% hObject    handle to ERB_Cambridge (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of ERB_Cambridge
set(handles.ERB_Cambridge,'Value',1);
set(handles.ERB_Munich,'Value',0);
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)

% --- Executes on button press in ERB_Munich.
function ERB_Munich_Callback(hObject, eventdata, handles)
% hObject    handle to ERB_Munich (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of ERB_Munich
set(handles.ERB_Cambridge,'Value',0);
set(handles.ERB_Munich,'Value',1);

playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)



% --- Executes on button press in Run.
function Run_Callback(hObject, eventdata, handles)
% hObject    handle to Run (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
playsound=1;
Plot_figure(hObject, eventdata, handles,playsound)


% --- Executes on selection change in popupmenu4.
function popupmenu4_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns popupmenu4 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu4


% --- Executes during object creation, after setting all properties.
function popupmenu4_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in temporalpos.
function temporalpos_Callback(hObject, eventdata, handles)
% hObject    handle to temporalpos (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of temporalpos


% --- Executes on button press in magnitude.
function magnitude_Callback(hObject, eventdata, handles)
% hObject    handle to magnitude (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of magnitude
set(handles.realpart,'Value',0)
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)

% --- Executes on button press in realpart.
function realpart_Callback(hObject, eventdata, handles)
% hObject    handle to realpart (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of realpart
set(handles.magnitude,'Value',0)
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)

% --- Executes on button press in showresponse.
function showresponse_Callback(hObject, eventdata, handles)
% hObject    handle to showresponse (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)






function cc_Callback(hObject, eventdata, handles)
% hObject    handle to cc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of cc as text
%        str2double(get(hObject,'String')) returns contents of cc as a double
if str2double(get(handles.cc,'String'))>4;  set(handles.cc,'String',4);end
if str2double(get(handles.cc,'String'))<-4;  set(handles.cc,'String',-4);end
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)


% --- Executes during object creation, after setting all properties.
function cc_CreateFcn(hObject, eventdata, handles)
% hObject    handle to cc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in asymfreq.
function asymfreq_Callback(hObject, eventdata, handles)
% hObject    handle to asymfreq (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of asymfreq
set(handles.peakfreqbutton,'Value',0);
set(handles.stapes,'Value',0);
%set(handles.Run,'Value',1)
% playsound=1;
% Plot_figure(hObject, eventdata, handles,playsound)
%Run_Callback(hObject, eventdata, handles)
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)

% --- Executes on button press in peakfreqbutton.
function peakfreqbutton_Callback(hObject, eventdata, handles)
% hObject    handle to peakfreqbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of peakfreqbutton
set(handles.asymfreq,'Value',0);
set(handles.stapes,'Value',0);
%set(handles.Run,'Value',1)
% playsound=1;
% Plot_figure(hObject, eventdata, handles,playsound)
%Run_Callback(hObject, eventdata, handles)
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)


% --- Executes on button press in stapes.
function stapes_Callback(hObject, eventdata, handles)
% hObject    handle to stapes (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of stapes
set(handles.asymfreq,'Value',0);
set(handles.peakfreqbutton,'Value',0);
playsound=0;
Plot_figure(hObject, eventdata, handles,playsound)


% --- Executes on selection change in loadsignals.
function loadsignals_Callback(hObject, eventdata, handles)
% hObject    handle to loadsignals (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns loadsignals contents as cell array
%        contents{get(hObject,'Value')} returns selected item from loadsignals
playsound=0;

Plot_figure(hObject, eventdata, handles,playsound)


% --- Executes during object creation, after setting all properties.
function loadsignals_CreateFcn(hObject, eventdata, handles)
% hObject    handle to loadsignals (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
